#define CONST_ARRAY_SIZE 13
//
//  DirectX vertex shader constant register file assignments
//  Each register is a 4-component floating point vector.
//  Any register can be freely changed OUTSIDE the shader.
//  Once in the shader, the registers are constant (hence the name).
//
//  For Vertex Shader version 1.1, we get only 96 constants.
//  For Vertex Shader version 2.0, we get at least 256 constants.
//
//  The following entries define the index into the constant register file
//  Each section is for a particular shader
//

#define _VS_CONSTANTS_H_

// mdw-2004-11-09: I added these two lines to debug the vertex and pixel shaders
// (please note: debugging vertex shaders forces software vertex processing which is slow)
// (even worse, debugging pixel shaders forces the reference driver which is SLOOOOOOW)

//#define DEBUG_VERTEX_SHADERS		1
//#define DEBUG_PIXEL_SHADERS		1

#define VS_CONST(val) val

#define WORLD_LIGHTING
//---------------------------------------------------------------------------------------------
//  These are shader constants are shared by a number of shaders
//---------------------------------------------------------------------------------------------
#define VS_InvScreenConstants VS_CONST(4)
#define VS_InvRenderTargetConstants VS_CONST(4)


#define VS_HaloScale VS_CONST(1)
#define VS_HaloColor VS_CONST(2)

//---------------------------------------------------------------------------------------------
//  Model drawing (models, shadows, halo effect, etc.)
//---------------------------------------------------------------------------------------------

#define VS_LightDir0 VS_CONST(5)
#define VS_LightDir1 VS_CONST(6)
#define VS_LightDir2 VS_CONST(7)
#define VS_LightColors0 VS_CONST(0)
#define VS_LightColors1 VS_CONST(1)
#define VS_LightColors2 VS_CONST(2)
#define VS_LightColors3 VS_CONST(3)
#define VS_ColorAmbient VS_CONST(8)

//---------------------------------------------------------------------------------------------
// The 2.0 vertex shaders pass matrices directly, which is more efficient and
// still leaves plenty of shader constants.
#define VS_BoneStart 9	// 152 constants (38 bones * 4 vectors per bone)
#define VS_SkinViewProgAff VS_CONST(123) // VS_BoneStart + 38*3
//---------------------------------------------------------------------------------------------

//Special case for 1 bone characters.
#define VS_Bone0M00 VS_BoneStart+0
#define VS_Bone0M10 VS_BoneStart+1
#define VS_Bone0M20 VS_BoneStart+2
#define VS_Bone0M30 VS_BoneStart+3


//---------------------------------------------------------------------------------------------
//  Constants for shaders that draw world models, particles, floor sprites, etc.
//---------------------------------------------------------------------------------------------
#define VS_XformM00 VS_CONST(0)
#define VS_XformM10 VS_CONST(1)
#define VS_XformM20 VS_CONST(2)
#define VS_XformM30 VS_CONST(3)
#define VS_ConstantColor VS_CONST(5)
#define VS_WorldPosOffset VS_CONST(5)

// These are set as a group of floats arranged in a float array called "directions" in light.cpp
#define VS_Normality VS_CONST(7)
#define VS_LightPos1 VS_CONST(8)
#define VS_LightPos2 VS_CONST(9)
#define VS_LightPos3 VS_CONST(10)
#define VS_LightRadius VS_CONST(11)
#define VS_LightFalloff VS_CONST(12)
#define VS_LightColor1 VS_CONST(13)
#define VS_LightColor2 VS_CONST(14)
#define VS_LightColor3 VS_CONST(15)
#define VS_LightColor4 VS_CONST(16)
#define VS_LightAmbient VS_CONST(17)
#define VS_RefMat1 VS_CONST(18)
#define VS_RefMat2 VS_CONST(19)
#define VS_PosScale VS_CONST(6)

#define VS_ParticleTime VS_CONST(5)
#define VS_ParticleDescListStart 7	//  48 constants (16 particle descriptors * 3 vectors each), beginning here


uniform highp vec4 c[CONST_ARRAY_SIZE];

highp mat4 getMat4( int idx )
{
return mat4( c[idx], c[idx+1], c[idx+2], c[idx+3] );
}
#define invScreenConstants c[VS_InvScreenConstants]

highp vec4 toScreen( highp vec4 pos )
{
return pos * invScreenConstants;
}
lowp vec4 applyLight( highp vec3 xformedNormal )
{
highp vec4 lightAmount =	xformedNormal.x * c[VS_LightDir0] +
xformedNormal.y * c[VS_LightDir1] +
xformedNormal.z * c[VS_LightDir2];

lightAmount = max( lightAmount, 0.0 );

lowp vec4 lightColor =	c[VS_ColorAmbient] +
lightAmount.x * c[VS_LightColors0] +
lightAmount.y * c[VS_LightColors1] +
lightAmount.z * c[VS_LightColors2] +
lightAmount.w * c[VS_LightColors3];

return min(lightColor, 1.0);
}

attribute mediump vec3 position;
attribute mediump vec3 normal;
attribute mediump vec2 texUV;

varying lowp vec4 blendColor;

varying mediump vec2 texCoord;

void main()
{
highp vec4 xformedPos =	c[VS_Bone0M00] * position.x +
c[VS_Bone0M10] * position.y +
c[VS_Bone0M20] * position.z +
c[VS_Bone0M30];

highp vec3
xformedNormal =	normal.x * c[VS_Bone0M00].xyz +
normal.y * c[VS_Bone0M10].xyz +
normal.z * c[VS_Bone0M20].xyz;

blendColor = applyLight(xformedNormal);

gl_Position = toScreen( xformedPos );
texCoord = texUV;
}
